# -*- coding:utf-8 -*-

"""
封装数据库操作, 自定义元类, 定义Model组件
"""

from mongoengine import *
from flask_backstage.mongoengine import db
from flask_login import UserMixin


class Menu(db.Document):
	"""
	后台用户
	:cvar name 菜单名
	:cvar role_name 角色字段名
	:cvar father 父级菜单名
	:cvar url 对应url链接
	:cvar permissions 菜单权限限制
	:cvar sort 菜单排序
	"""
	name = StringField(required=True, unique=True)
	father = StringField()
	url = StringField()
	permissions = ListField()
	sort = IntField()


class Role(db.Document):
	"""
	角色
	:cvar name 角色名
	:cvar value 值
	"""
	name = StringField(required=True)
	value = IntField(required=True)

class User(db.DynamicDocument, UserMixin):
	username = StringField(required=True)
	password = BinaryField(required=True)
	role = IntField(required=True)

	def get_role_name(self):
		r = Role.objects(value=self.role).first()
		if r:
			return r.name
		else:
			return None



# -*- coding:utf-8 -*-
from flask_wtf import FlaskForm
from flask import request
from flask_login import current_user
from collections import Iterable


def expose(url='/', methods=('GET',)):
    """动态生成路由"""

    def wrap(f):
        if not hasattr(f, '_urls'):
            f._urls = []
        f._urls.append((url, methods))
        return f

    return wrap


def get_field_value(model, name):
    """获取model对应字段的值"""
    return getattr(model, name)


def is_form_submitted():
    return request and request.method in ('PUT', 'POST')


class MyForm(FlaskForm):
    """重写populate_obj方法"""

    def populate_obj(self, obj):
        for name, field in self._fields.items():
            if request.form.get(name) is None or request.form.get(name) is '':
                continue
            field.populate_obj(obj, name)


def get_form(model, converter, field_args, base_class=MyForm):
    """返回一个表单类"""
    properties = sorted(((k, v) for k, v in model._fields.items()), key=lambda v: v[1].creation_counter)  # 对字典进行排序
    field_dict = {}
    for name, p in properties:
        field = converter.convert(model, p, field_args)  # 调用convert方法，返回对应的field
        if field is not None:
            field_dict[name] = field
    field_dict['model_class'] = model
    return type(model.__name__ + 'Form', (base_class,), field_dict)  # 构造表单类


def cadmin_create_required(role=None):
    """
    cadmin权限装饰器
    :param role: 角色值
    :return:
    """

    if not isinstance(current_user.role, int):
        raise Exception('role参数必须是整数')
    if isinstance(role, int):
        if current_user.role >= role:
            return True
        else:
            return False
    if isinstance(role, Iterable):
        if current_user.role in role:
            return True
        else:
            return False



# -*- coding:utf-8 -*-
from flask_mongoengine.wtf import orm
from wtforms import fields as f, validators
from .widgets import DateTimePickerField


class CustomModelConverter(orm.ModelConverter):
	def __init__(self, view):
		super(CustomModelConverter, self).__init__()

		self.view = view

	def _get_field_override(self, name):
		form_overrides = getattr(self.view, 'form_overrides', None)

		if form_overrides:
			return form_overrides.get(name)

		return None

	def convert(self, model, field, field_args):
		kwargs = {
			'label': getattr(field, 'verbose_name', field.name),
			'description': getattr(field, 'help_text', None) or '',
			'validators': getattr(field, 'validators', None) or [],
			'filters': getattr(field, 'filters', None) or [],
			'default': field.default,
		}
		if field_args:
			kwargs.update(field_args)

		if kwargs['validators']:
			# Create a copy of the list since we will be modifying it.
			kwargs['validators'] = list(kwargs['validators'])

		if field.required:
			kwargs['validators'].append(validators.InputRequired())
		else:
			kwargs['validators'].append(validators.Optional())

		ftype = type(field).__name__

		if field.choices:
			kwargs['choices'] = field.choices

			if ftype in self.converters:
				kwargs["coerce"] = self.coerce(ftype)
			multiple_field = kwargs.pop('multiple', False)
			radio_field = kwargs.pop('radio', False)
			if multiple_field:
				return f.SelectMultipleField(**kwargs)
			if radio_field:
				return f.RadioField(**kwargs)
			return f.SelectField(**kwargs)

		ftype = type(field).__name__

		if hasattr(field, 'to_form_field'):
			return field.to_form_field(model, kwargs)

		override = self._get_field_override(field.name)
		if override:
			return override(**kwargs)

		if ftype in self.converters:
			return self.converters[ftype](model, field, kwargs)

	@orm.converts('DateTimeField')
	def conv_DateTime(self, model, field, kwargs):
		return DateTimePickerField(**kwargs)



# -*- coding:utf-8 -*-
from flask_mongoengine import MongoEngine
from flask_bcrypt import Bcrypt
from flask_wtf.csrf import CSRFProtect

#数据库
db = MongoEngine()

# 安全组件
csrf = CSRFProtect()
bcrypt = Bcrypt()

class Admin(object):
	def __init__(self, app=None):
		if app is not None:
			self.init_app(app)

	def init_app(self, app):
		db.init_app(app)  # 数据库
		# 安全组件
		csrf.init_app(app)  # 全站开通csrf保护,配置文件需要启用
		bcrypt.init_app(app)  # 密码加密组件
		from .views.admin import admin
		app.register_blueprint(admin, url_prefix='/admin')
		from .models import Menu, User, Role
		from flask_login import LoginManager
		login_manager = LoginManager()
		login_manager.init_app(app)  # 用于用户的登录，登出和登录访问控制

		login_manager.login_view = "admin.login"

		@login_manager.user_loader
		def load_user(user_id):
			return User.objects(id=user_id).first()

		from flask_login import current_user
		admin = User.objects(role=300).first()
		role = Role.objects(value=300).first()
		if not admin:
			user = User()
			user.username = 'admin'
			user.password = bcrypt.generate_password_hash('123456')
			user.role = 300
			user.save()
		if not role:
			role = Role()
			role.name = '超级管理员'
			role.value = 300
			role.save()

		@app.context_processor
		def menus_list():
			"""
			从数据库查询出用户有权限查看的菜单
			:return:
			"""
			if current_user.is_authenticated:
				roots = Menu.objects(father='无').order_by('sort')
				menus = []
				for r in roots:
					accessible = current_user.role in r.permissions
					if accessible:
						menus.append({'my': r, 'child': build_child_list(r)})
					else:
						continue
				return dict(menus_list=menus)
			else:
				return dict()

		def build_child_list(r):
			childs = Menu.objects(father=r.name).order_by('sort')
			if childs:
				menus = []
				for c in childs:
					accessible = current_user.role in c.permissions
					if accessible:
						menus.append({'my': c, 'child': build_child_list(c)})
					else:
						continue
				return menus
			else:
				return False



# -*- coding:utf-8 -*-
"""
创建视图函数工具类
"""
from flask import Blueprint, request, render_template, redirect, url_for, abort
from .helps import expose, get_field_value, get_form, is_form_submitted
from .create_form import CustomModelConverter
from flask_login import login_required


class CAdmin(object):
    def __init__(self, app=None):
        self.app = app
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app = app

    def add_view(self, view):
        """注册蓝图"""
        if self.app is not None:
            self.app.register_blueprint(view.create_blueprint())


class BaseView(object):
    """基础视图类:cvar create_blueprint方法返回生成的蓝图"""

    def __init__(self, endpoint=None, static_folder=None, template_folder=None, url_prefix=None):
        self.endpoint = endpoint
        self.static_folder = static_folder
        self.template_folder = template_folder
        self.url_prefix = url_prefix
        self._urls = []
        self.blueprint = None

        for p in dir(self):
            """给_urls赋值"""
            attr = getattr(self, p)

            if hasattr(attr, '_urls'):
                """有_urls的说明就是被expose注册的路由"""
                for url, methods in attr._urls:
                    self._urls.append((url, p, methods))

    def create_blueprint(self):
        if not self.endpoint:
            self.endpoint = self.__class__.__name__.lower()
        self.blueprint = Blueprint(self.endpoint, __name__, template_folder=self.template_folder, static_folder=self.static_folder, url_prefix=self.url_prefix)
        for url, name, methods in self._urls:
            """添加路由"""
            self.blueprint.add_url_rule(url, name, getattr(self, name), methods=methods)
        return self.blueprint


class BaseModelView(BaseView):
    """基础modelView"""
    column_labels = None  # 对应字段的标题显示，例子dict(title="标题", content="内容", time="发布时间", image='图片')
    name = None  #  所操作的model的名字，例子name='文章'，则前端显示的就是文章列表，添加文章，编辑文章这样的字样
    image_list = []  #  说明那个字段是图片，列表显示的时候就会以图片形式显示，而不是图片路径
    list_templates = 'admin/list.html'  # list模板路径
    create_templates = 'admin/create.html'  # create模板路径
    edit_templates = 'admin/edit.html'  #  edit模板路径
    can_create = True  #  控制是否有添加功能
    can_edit = True  #  控制是否有编辑功能
    can_delete = True  #  控制是否有删除功能

    def __init__(self, model, endpoint=None, static_folder=None, template_folder=None, url_prefix=None):
        self.model = model
        if not url_prefix:
            url_prefix = '/' + self.model.__name__.lower()
        super(BaseModelView, self).__init__(endpoint=endpoint, static_folder=static_folder, template_folder=template_folder, url_prefix=url_prefix)

    def _get_model_fields(self, model=None):
        """通过model获取model的字段名"""
        if model is None:
            model = self.model

        return sorted(model._fields.items(), key=lambda n: n[1].creation_counter)

    def scaffold_form(self):
        """调用get_form生成表单类"""
        form_class = get_form(self.model, CustomModelConverter(self), {})
        return form_class

    def create_form(self):
        return self.scaffold_form()

    @expose('/list/')
    @login_required
    def list_view(self):
        """列表"""
        if not self.is_accessible():
            abort(403)
        if not self.create_required():
            self.can_create = False
        if not self.edit_required():
            self.can_edit = False
        if not self.delete_required():
            self.can_delete = False
        fields = self._get_model_fields(self.model)
        page = request.args.get('page')  # 分页
        if not page:
            page = 1
        else:
            page = int(page)
        data = self.model.objects.paginate(page=page, per_page=10)
        return render_template(
            self.list_templates, data=data, name=self.name,
            fields=fields, get_field_value=get_field_value,
            column_labels=self.column_labels, url_prefix=self.url_prefix, image_list=self.image_list,
            can_create=self.can_create, can_edit=self.can_edit, can_delete=self.can_delete
        )

    @expose('/create/', methods=('GET', 'POST'))
    @login_required
    def create_view(self):
        """添加保存"""
        if not self.is_accessible():
            abort(403)
        if not self.create_required():
            abort(403)
        form = self.scaffold_form()()  # 调用scaffold_form生成表单类，并实例化这个表单
        if is_form_submitted():
            model = self.model()
            form.populate_obj(model)
            model.save()
            return redirect(url_for(".list_view"))
        return render_template(
            self.create_templates, name=self.name, form=form,
            column_labels=self.column_labels, create_url=url_for(".create_view"))

    @expose('/edit/', methods=('GET', 'POST'))
    @login_required
    def edit_view(self):
        """编辑保存"""
        if not self.is_accessible():
            abort(403)
        if not self.edit_required():
            abort(403)
        form = self.scaffold_form()()  # 调用scaffold_form生成表单类，并实例化这个表单
        if is_form_submitted():
            model_id = request.args.get('id')
            model = self.model.objects(id=model_id).first()
            form.populate_obj(model)
            model.save()
            return redirect(url_for(".list_view"))
        else:
            model_id = request.args.get('id')
            model = self.model.objects(id=model_id).first()
            form = self.scaffold_form()(obj=model)
            return render_template(
                self.edit_templates, name=self.name, form=form,
                column_labels=self.column_labels, edit_url=url_for(".edit_view"), id=model_id)

    @expose('/delete/')
    @login_required
    def delete_view(self):
        """删除"""
        if not self.is_accessible():
            abort(403)
        if not self.delete_required():
            abort(403)
        model_id = request.args.get('id')
        model = self.model.objects(id=model_id).first()
        model.delete()
        return redirect(url_for(".list_view"))

    def create_required(self):
        """
        添加操作的权限控制，通过重写这个函数达到对添加操作的权限控制
        :return:
        """
        return self.can_create

    def edit_required(self):
        """
        编辑操作的权限控制，通过重写这个函数达到对编辑操作的权限控制
        :return:
        """
        return self.can_edit

    def delete_required(self):
        """
        删除操作的权限控制，通过重写这个函数达到对删除操作的权限控制
        :return:
        """
        return self.can_delete

    def is_accessible(self):
        """
        所有操作的统一权限控制，通过重写这个函数达到对所有操作的权限控制
        :return:
        """
        return True



# -*- coding:utf-8 -*-
"""
子目录统一使用带/的url风格,如:@apps.route('/projects/'), methods默认是get和head
Blueprint 注册, 使用范例:
frontend = Blueprint('frontend',  # 注册蓝图名称
        __name__,
        template_folder='/opt/auras/templates/',   # 指定模板路径
        static_folder='/opt/auras/flask_bootstrap/static/',# 指定静态文件路径
                   )
"""

from flask import Blueprint, render_template, redirect, url_for, flash, request
from ..models import Menu, Role, User
from ..forms import MenuForm, RoleForm, LoginForm, UserForm
from flask_login import login_required, logout_user, login_user
from ..utils.helps import admin_create_required
from .. import bcrypt

admin = Blueprint('admin', __name__, template_folder='../templates', static_folder='../static')

admin_required = admin_create_required(300)



@admin.route('/', methods=('GET', 'POST'))
def login():
	"""
	登录
	:return:
	"""
	form = LoginForm()
	if form.validate_on_submit():
		login_user(form.get_user())
		return redirect(url_for('admin.menu_list'))
	return render_template('admin/login.html', form=form)


@admin.route('/login_out/', methods=('GET', 'POST'))
@login_required
def login_out():
	"""
	退出登录
	:return:
	"""
	logout_user()
	return redirect(url_for('admin.login'))


@admin.route('/menu_list/', methods=('GET', 'POST'))
@login_required
@admin_required
def menu_list():
	"""
	进入菜单列表
	:return:
	"""
	menus = Menu.objects
	return render_template('admin/menu_list.html', menus=menus)


@admin.route('/menu_add/', methods=('GET', 'POST'))
@login_required
@admin_required
def menu_add():
	"""
	添加菜单
	:return:
	"""
	form = MenuForm()
	menus = Menu.objects
	roles = Role.objects
	form.father.choices = [('无', '无')] + [(m.name, m.name) for m in menus]
	form.permissions.choices = [(0, '默认权限')] + [(r.value, r.name) for r in roles]
	return render_template('admin/menu_add.html', form=form)


@admin.route('/menu_save/', methods=('GET', 'POST'))
@login_required
@admin_required
def menu_save():
	"""
	保存菜单
	:return:
	"""
	form = MenuForm()
	menu = Menu()
	form.populate_obj(menu)
	permissions = request.form.get('permissions').split(",")[0:-1]
	permissions = list(map(int, permissions))
	menu.permissions = permissions
	menu.save()
	return redirect(url_for('admin.menu_list'))


@admin.route('/menu_edit/<string:mid>/', methods=('GET', 'POST'))
@login_required
@admin_required
def menu_edit(mid):
	"""
	编辑菜单
	:param mid:
	:return:
	"""
	menu = Menu.objects(id=mid).first()
	roles = Role.objects
	form = MenuForm()
	menus = Menu.objects
	form.father.choices = [('无', '无')] + [(m.name, m.name) for m in menus]
	form.name.data = menu.name
	form.father.data = menu.father
	form.url.data = menu.url
	form.permissions.choices = [(0, '默认权限')] + [(r.value, r.name) for r in roles]
	form.sort.data = menu.sort
	permissions = ",".join(list(map(str, menu.permissions))) + ","

	return render_template('admin/menu_edit.html', form=form, menu=menu, permissions=permissions)


@admin.route('/menu_editsave/', methods=('GET', 'POST'))
@login_required
@admin_required
def menu_editsave():
	"""
	保存编辑的菜单
	:return:
	"""
	mid = request.form.get('mid')
	form = MenuForm()
	menu = Menu.objects(id=mid).first()
	permissions = request.form.get('permissions').split(",")[0:-1]
	permissions = list(map(int, permissions))
	form.populate_obj(menu)
	menu.permissions = permissions
	menu.save()
	return redirect(url_for('admin.menu_list'))


@admin.route('/menu_del/<string:mid>/', methods=('GET', 'POST'))
@login_required
@admin_required
def menu_del(mid):
	"""
	删除菜单
	:param mid:
	:return:
	"""
	menu = None
	if mid:
		menu = Menu.objects(id=mid).first()
	if not menu:
		flash(u'对不起，角色不存在或已经删除')
	else:
		menu.delete()
		flash(u'删除角色成功')
	return redirect(url_for('admin.menu_list'))


@admin.route('/role_list', methods=('GET', 'POST'))
@login_required
@admin_required
def role_list():
	"""
	角色列表
	:return:
	"""
	roles = Role.objects
	return render_template('admin/role_list.html', roles=roles)


@admin.route('/role_add/', methods=('GET', 'POST'))
@login_required
@admin_required
def role_add():
	"""
	添加角色
	:return:
	"""
	form = RoleForm()
	return render_template('admin/role_add.html', form=form)


@admin.route('/role_save/', methods=('GET', 'POST'))
@login_required
@admin_required
def role_save():
	"""
	角色保存
	:return:
	"""
	form = RoleForm()
	role = Role()
	form.populate_obj(role)
	role.save()
	return redirect(url_for('admin.role_list'))


@admin.route('/role_edit/<string:rid>/', methods=('GET', 'POST'))
@login_required
@admin_required
def role_edit(rid):
	"""
	编辑角色
	:param rid:
	:return:
	"""
	role = Role.objects(id=rid).first()
	form = RoleForm()
	form.name.data = role.name
	form.value.data = role.value
	return render_template('admin/role_edit.html', form=form, role=role)


@admin.route('/role_editsave/', methods=('GET', 'POST'))
@login_required
@admin_required
def role_editsave():
	"""
	保存角色编辑
	:return:
	"""
	rid = request.form.get('rid')
	form = RoleForm()
	role = Role.objects(id=rid).first()
	form.populate_obj(role)
	role.save()
	return redirect(url_for('admin.role_list'))


@admin.route('/role_del/<string:rid>/', methods=('GET', 'POST'))
@login_required
@admin_required
def role_del(rid):
	"""
	删除角色
	:param rid:
	:return:
	"""
	role = None
	if rid:
		role = Role.objects(id=rid).first()
	if not role:
		flash(u'对不起，角色不存在或已经删除')
	else:
		role.delete()
		flash(u'删除角色成功')
	return redirect(url_for('admin.role_list'))


@admin.route('/user_list', methods=('GET', 'POST'))
@login_required
@admin_required
def user_list():
	"""
	用户列表
	:return:
	"""
	users = User.objects
	return render_template('admin/user_list.html', users=users)


@admin.route('/user_add/', methods=('GET', 'POST'))
@login_required
@admin_required
def user_add():
	"""
	添加用户
	:return:
	"""
	form = UserForm()
	roles = Role.objects
	return render_template('admin/user_add.html', form=form, roles=roles)


@admin.route('/user_save/', methods=('GET', 'POST'))
@login_required
@admin_required
def user_save():
	"""
	用户保存
	:return:
	"""
	user = User()
	username = request.form.get('username')
	password = request.form.get('password')
	role = request.form.get('role')
	if username:
		user.username = username
	if password:
		user.password = bcrypt.generate_password_hash(password)
	if role:
		user.role = int(role)
	user.save()
	return redirect(url_for('admin.user_list'))


@admin.route('/user_edit/<string:uid>/', methods=('GET', 'POST'))
@login_required
@admin_required
def user_edit(uid):
	"""
	编辑用户
	:param rid:
	:return:
	"""
	user = User.objects(id=uid).first()
	roles = Role.objects
	form = UserForm()
	form.username.data = user.username
	return render_template('admin/user_edit.html', form=form, user=user, roles=roles)


@admin.route('/user_editsave/', methods=('GET', 'POST'))
@login_required
@admin_required
def user_editsave():
	"""
	保存用户编辑
	:return:
	"""
	uid = request.form.get('uid')
	user = User.objects(id=uid).first()
	username = request.form.get('username')
	password = request.form.get('password')
	role = request.form.get('role')
	if username:
		user.username = username
	if password:
		user.password = bcrypt.generate_password_hash(password)
	if role:
		user.role = int(role)
	user.save()
	return redirect(url_for('admin.user_list'))


@admin.route('/user_del/<string:uid>/', methods=('GET', 'POST'))
@login_required
@admin_required
def user_del(uid):
	"""
	删除用户
	:param uid:
	:return:
	"""
	user = None
	if uid:
		user = User.objects(id=uid).first()
	if not user:
		flash(u'对不起，角色不存在或已经删除')
	else:
		user.delete()
		flash(u'删除角色成功')
	return redirect(url_for('admin.user_list'))



from flask import Flask
from flask_backstage import Admin

app = Flask(__name__)

# 添加csrf保护配置
app.config['WTF_CSRF_ENABLED'] = True
app.config['SECRET_KEY'] = '0\xda\xb3+xAj\x88\x98\x00\x02\xcaR\x9d\xb5QD\x0c\x1aX"\x89{S'

admin = Admin(app)

app.run()


# -*- coding:utf-8 -*-
from wtforms import widgets, fields


class DateTimePickerWidget(widgets.TextInput):
	data_role = 'datatime'

	def __call__(self, field, **kwargs):
		return super(DateTimePickerWidget, self).__call__(field, **kwargs)


class DateTimePickerField(fields.StringField):
	widget = DateTimePickerWidget()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(name='flask_backstage',
      version='0.1.7',
      description='A backstage framework with very few code.',
      author='pengwei',
      author_email='1300896173@qq.com',
      url='https://github.com/huanglong123/flask-backstage',
      packages=find_packages(),
      include_package_data=True,
      license='MIT',
      platforms='any',
      install_requires=[
          'Flask>=0.7',
          'Flask-WTF',
          'Flask-Bcrypt',
          'Flask-Login',
          'flask-mongoengine',
          'Flask-WTF'
      ],
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Web Environment',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Software Development :: Libraries :: Python Modules',
                   'Programming Language :: Python :: 3.5',
                   ],
      )



# -*- coding:utf-8 -*-
"""
表单类
"""
from flask_wtf import FlaskForm
from wtforms import StringField, IntegerField, SelectField, PasswordField
from wtforms import validators
from ..models import User
from .. import bcrypt


class MenuForm(FlaskForm):
	name = StringField('菜单名')
	father = SelectField('父级菜单')
	url = StringField('菜单地址')
	permissions = SelectField('菜单权限限制')
	sort = IntegerField('菜单排序')


class RoleForm(FlaskForm):
	name = StringField('角色名')
	value = IntegerField('值')


class UserForm(FlaskForm):
	username = StringField('用户名')
	password = PasswordField('密码')
	role = SelectField('角色')


class LoginForm(FlaskForm):
	username = StringField('用户名')
	password = PasswordField('密码')

	def get_user(self):
		user = User.objects(username=self.username.data).first()
		return user

	def validate_password(self, field):
		user = self.get_user()
		if user:
			if not user.password:
				raise validators.ValidationError('密码未设置')
			if not bcrypt.check_password_hash(user.password, self.password.data):
				raise validators.ValidationError('密码错误')
		else:
			raise validators.ValidationError('用户不存在')



# -*- coding:utf-8 -*-

"""
网站权限相关的程序
"""

from functools import wraps
from flask import abort
from flask_login import current_user
from collections import Iterable

def admin_create_required(role=None):
    """
    admin权限装饰器
    :param role: 角色值
    :return:
    """
    if role is None:
        raise Exception('role参数是必填参数')

    def required(f):
        @wraps(f)
        def decorated_function(*args, **kwargs):
            if not isinstance(current_user.role, int):
                raise Exception('role参数必须是整数')
            if isinstance(role, int):
                if current_user.role >= role:
                    return f(*args, **kwargs)
                else:
                    abort(403)
            if isinstance(role, Iterable):
                if current_user.role in role:
                    return f(*args, **kwargs)
                else:
                    abort(403)
        return decorated_function
    return required